import React, { useRef, useEffect, useState, useContext, forwardRef, createContext, useMemo } from 'react'
import { ConfigProvider, message, Modal, Button } from 'antd'
import Generator from 'fr-generator'
import { settings, baseSettings, globalSettings } from './config'
import widgets from '../widgets'
import { saveTableConfig, GetTableConfigJson } from '../../apis/process'
import FormRender from '../FormRender'

export const GlobalStore = createContext(null)

const FormDesigner = (props, ref) => {

  const { tableName } = props

  const { getPrefixCls } = useContext(ConfigProvider.ConfigContext)
  const prefixCls = getPrefixCls('pandaXform')
  const [visible, setVisible] = useState(false)
  const [schema, setSchema] = useState({})
  const designerRef = useRef(null)
  const formRenderRef = useRef(null)

  const settingsParent = useMemo(() => {
    let settingsParent = []
    settings.forEach(v => {
      v.widgets.forEach(s => {
        s.schema = { ...s.schema, tableNameParent: tableName }
      })
      settingsParent.push(v)
    })
    return settingsParent
  }, [settings, tableName])

  const getTableField = () => {
    console.log('setSchema', settings)
  }

  const getTableConfig = async () => {
    const { code, data, msg } = await GetTableConfigJson({ tableName })
    if (code === 0) {
      if (typeof data === 'string') {
        setTimeout(() => {
          designerRef?.current?.setValue(JSON.parse(data))
          // setSchema(JSON.parse(data))
        }, 0)
      }
    } else {
      message.error(msg)
    }
  }

  useEffect(() => {
    getTableConfig()
  }, [])

  const extraButtons = [
    false,
    true,
    true,
    true,
    {
      text: '预览',
      onClick: (e) => {
        setVisible(true)
        setSchema(designerRef.current.getValue())
      }
    },
    {
      text: '提交',
      onClick: async (value) => {
        const errors = designerRef.current.getErrorFields()
        const json = designerRef.current.getValue()
        if (errors.length) {
          return message.error('请按照提示完善表单内容')
        }
        // console.log('json.properties', json.properties)
        // if (json.properties && Object.keys(json.properties).includes(v => !v.includes('分组名称'))) {
        //   return message.info('包含未分组形态!')
        // }
        const { code, data, msg } = await saveTableConfig(json)
        if (code === 0) {
          message.info('保存成功')
        } else {
          message.error(msg)
        }
      }
    }
  ]

  const testSubmit = async () => {
    const { formValue, errors } = await formRenderRef?.current?.getValues()
    if (errors.length) {
      return message.error('表单校验未通过!')
    }
    console.log('formValue', formValue)
  }

  return (
    <div className={prefixCls} style={{ width: '100%', height: '100%', background: 'white' }}>
      <div style={{ height: '100%' }}>
        <Generator
          configProvider={{ prefixCls: prefixCls }}
          mapping={{
            object: 'Header',
          }}
          validation={true}
          ref={designerRef}
          extraButtons={extraButtons}
          widgets={widgets}
          settings={settingsParent}
          commonSettings={baseSettings}
          globalSettings={globalSettings}
        />
        <Modal
          title={tableName}
          width='80%'
          bodyStyle={{ height: '700px', overflow: 'auto' }}
          onOk={() => setVisible(false)}
          onCancel={() => setVisible(false)}
          visible={visible}
          getContainer={false}
          destroyOnClose
          footer={
            <div>
              <Button onClick={() => setVisible(false)}>取消</Button>
              <Button onClick={() => setVisible(false)}>确定</Button>
              <Button onClick={testSubmit}>测试提交</Button>
            </div>
          }
        >
          <FormRender ref={formRenderRef} schema={schema} />
        </Modal>
      </div>
    </div>
  )

}

export default forwardRef(FormDesigner)